require 'studio_game/philosopher'
require 'studio_game/spec_helper'
require 'studio_game/epiphany_trove'

module StudioGame
  describe Philosopher do
    before do
      @initial_health = 150
      @player = Philosopher.new("camus", @initial_health)
      $stdout = StringIO.new
    end
  
    it "can be created from a CSV string" do
      player = Philosopher.from_csv("Camus,150")
    
      player.name.should == "Camus"
      player.health.should == 150
    end
  
    it "has a capitalized name" do
      @player.name.should == "Camus"
    end
  
    it "has an initial health" do
      @player.health.should == 150
    end
  
    it "has a string representation" do
      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))
      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))
    
      @player.to_s.should == "\nI'm Camus with health = 150, points = 100, and score = 250."
    end
  
    it "computes a score as the sum of its health and points" do
      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))
      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))
    
      @player.score.should == 250
    end
  
    it "increases health by 15 when w00ted" do
      @player.w00t
    
      @player.health.should == (@initial_health + 15)
    end
  
    it "decreases health by 20 when blammed" do
      @player.blam
    
      @player.health.should == (@initial_health - 20)
    end
  
    context "with a health greater than 100" do
      before do
        @player = Philosopher.new("camus", 150)
      end
    
      it "is strong" do 
        @player.should be_strong
      end
    end
  
    it "computes points as the sum of all epiphany points" do
      @player.points.should == 0

      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))

      @player.points.should == 50

      @player.discovered_epiphanies(Epiphany.new(:the_problem_of_induction, 200))

      @player.points.should == 250

      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))

      @player.points.should == 300
    end
  
    it "yields each discovered epiphany and its total points" do
      @player.discovered_epiphanies(Epiphany.new(:a_monad, 100))
      @player.discovered_epiphanies(Epiphany.new(:a_monad, 100))
      @player.discovered_epiphanies(Epiphany.new(:an_indubitable_truth, 50))
      @player.discovered_epiphanies(Epiphany.new(:that_the_unexamined_life_is_not_worth_living, 5))
      @player.discovered_epiphanies(Epiphany.new(:that_the_unexamined_life_is_not_worth_living, 5))
      @player.discovered_epiphanies(Epiphany.new(:that_the_unexamined_life_is_not_worth_living, 5))
      @player.discovered_epiphanies(Epiphany.new(:that_the_unexamined_life_is_not_worth_living, 5))
      @player.discovered_epiphanies(Epiphany.new(:that_the_unexamined_life_is_not_worth_living, 5))

      yielded = []
      @player.each_discovered_epiphany do |epiphany|
        yielded << epiphany
      end

      yielded.should == [
        Epiphany.new(:a_monad, 200),
        Epiphany.new(:an_indubitable_truth, 50),
        Epiphany.new(:that_the_unexamined_life_is_not_worth_living, 25)
     ]
    end
  
    context "with a health less than 100" do
      before do
        @player = Philosopher.new("camus", 100)
      end
    
      it "is weak" do
        @player.should be_weak
      end
    end
  
    context "with a health greater than 0" do
      before do
        @player = Philosopher.new("camus", 100)
      end
    
      it "is alive" do
        @player.should be_alive
      end
    end
      
    context "in a collection of players" do
      before do
        @player1 = Philosopher.new("moe", 100)
        @player2 = Philosopher.new("larry", 200)
        @player3 = Philosopher.new("curly", 300)

        @players = [@player1, @player2, @player3]
      end

      it "is sorted by decreasing score" do
        @players.sort.should == [@player3, @player2, @player1]
      end
    end
  end
end